"use strict"

const { PacketSpinel97, SpinelRx } = require('./spinel.js');

module.exports = function (RED) {

    function SpinelCreatePacket(config) {

        RED.nodes.createNode(this, config);
        var node = this;

        node.on('input', function (msg, send, done) {

            let packet = new PacketSpinel97(msg.payload);
            msg.payload = packet.GetBuffer(false);
            msg.hexString = packet.GetHex(",");
            msg.requestSIG = packet.SIG;
            
            send = send || function() { node.send.apply(node, arguments) };
            send(msg);
            
            if (done) done();

        });

    }
    RED.nodes.registerType("Spinel Tx", SpinelCreatePacket);

    function SpinelParserPacket(config) {

        RED.nodes.createNode(this, config);
        var node = this;
        this.format = config.format;
        var receiver = new SpinelRx({allowInvalid: false});

        try {
            node.on('input', function (msg, send, done) {

                receiver.onOk = (packet, err) => {
                    send = send || function() { node.send.apply(node, arguments) };
                    switch (this.format) {
                        case "bufferPacket": 
                            msg.payload = packet.buffer;
                            break;
                        case "bufferData": 
                            msg.payload = packet.object.DATA;
                            break;
                        case "string": 
                            msg.payload = packet.string;
                            break;
                        case "object": 
                        default:
                            msg.payload = packet.object;
                            break;
                    }
                    if (err) {
                        msg.error = err;
                        node.warn(`Error in ${err}!`);
                    }
                    send(msg);
                };
                receiver.newRxData(msg.payload);
                if (done) done();

            });
        } catch (error) {
            node.warn(error);
        }

    }
    RED.nodes.registerType("Spinel Rx", SpinelParserPacket);

}